/**
 * Created by Harris on 16/4/14.
 */

define("player", ["player.audio", "player.video", "jquery"], function (require, exports, module) {
    var $ = require("jquery");

    var Player = function (opts) {
        var o = this.opts = $.extend({}, Player.DEF, opts);

        if (!o.videoId) {
            throw "need video Id to play!";
        }

        this.$element = $(o.element);

        if (o.type == Player.MEDIA_TYPE.AUDIO) {
            this.playerType = 'audio'
        } else if (o.type == Player.MEDIA_TYPE.VIDEO) {
            this.playerType = 'video';
        }
        this._events = {};
        this._getVideoInfo();
    };

    Player.prototype = {
        constructor: Player,

        initPlayer: function () {
            var options = $.extend({}, this.opts.playerConf, {
                mediaSRC: this.mediaSRC,
                mediaType: this.canPlayType,
                parentNode: this.$element
            });
            
            this.player = new (require("player." + this.playerType))(options);
        },
        
        _mediaTest: function (type) {
            var playerType = this.playerType,
                testEl = this.testEl || (this.testEl = document.createElement(playerType));
            if (type == "m3u8") {
                type = "application/vnd.apple.mpegURL"
            } else {
                type = playerType + "/" + type;
            }
            
            return testEl.canPlayType && testEl.canPlayType(type) && type;
        },

        _getVideoInfo: function () {
            var videoId = this.opts.videoId,
                self = this;

            
            $.ajax({
                url: "http://s.video.sina.com.cn/video/h5play?video_id=" + videoId,
                method: 'GET',
                dataType: 'jsonp',
                success: function (result) {
                    if (result && result.code == "1") {
                        var canPlayType = false,
                            videos = result.data.videos,
                            firstCanPlay;
                        for (var _name in videos) {
                            canPlayType = self._mediaTest(_name);
                            if (canPlayType && !firstCanPlay) {
                                firstCanPlay = _name;
                                self.canPlayType = canPlayType;
                                break;
                            }
                        }

                        if (firstCanPlay) {
                            var media = videos[firstCanPlay],
                                url = "";
                            for (var key in media) {
                                var data = media[key];
                                url = data.file_api + "?vid=" + data.file_id;
                                break;
                            }

                            self.mediaSRC = url;
                            self.initPlayer();
                        } else {
                            self.trigger("error", {
                                code: -1, //无法获取可播放的资源,
                                result: result
                            })
                        }
                    } else {
                        self.trigger("error", {
                            code: -9999, //接口错误,
                            result: result
                        })
                    }
                }
            })
        },
        on: function (type, fn) {
            if ( !this._events[type] ) {
                this._events[type] = [];
            }

            this._events[type].push(fn);
        },

        off: function (type, fn) {
            if ( !this._events[type] ) {
                return;
            }

            var index = this._events[type].indexOf(fn);

            if ( index > -1 ) {
                this._events[type].splice(index, 1);
            }
        },

        trigger: function (type) {
            if ( !this._events[type] ) {
                return;
            }

            var i = 0,
                l = this._events[type].length;

            if ( !l ) {
                return;
            }

            for ( ; i < l; i++ ) {
                this._events[type][i].apply(this, [].slice.call(arguments, 1));
            }
        }
    };

    Player.MEDIA_TYPE = {
        AUDIO: 2,
        VIDEO: 3
    };

    Player.DEF = {
        videoId: '',
        type: Player.MEDIA_TYPE.AUDIO,
        element: '#player',
        playerConf: {}
    }
    
    module.exports = Player;

});

define("player.audio", ["jquery"], function (require, exports, module) {
    "use strict";
    var $ = require("jquery");

    var TPL_ROOT = '<div class="player audio">'
        + '            <div class="inner">'
        + '                <div class="opts-tool J_btn">'
        + '                    <i class="icons icon-audio-play J_play_status"></i>'
        + '                </div>'
        + '                <div class="main J_progress_panel needsclick">'
        + '                    <div class="times">'
        + '                        <span class="played J_played_time">--:--</span>'
        + '                        <span class="total J_total_time">--:--</span>'
        + '                    </div>'
        + '                    <div class="audio-progress">'
        + '                        <div class="loaded J_loaded_progress" style="width: 0;"></div>'
        + '                        <div class="played J_played_progress" style="width: 0;"></div>'
        + '                    </div>'
        + '                </div>'
        + '            </div>'
        + '        </div>';

    var AudioPlayer = function (opts) {
        var o = this.opts = $.extend({}, AudioPlayer.DEF, opts);
        if (!o.mediaSRC || o.mediaSRC == "") {
            throw "need music url to init player";
        }

        var $root = this.$root = $(TPL_ROOT);
        this.$parentNode = $(o.parentNode);
        this.$parentNode.append(this.$root);

        this.$playedTime = $root.find('.J_played_time');
        this.$totalTime = $root.find('.J_total_time');
        this.$loadedProgress = $root.find('.J_loaded_progress');
        this.$playedProgress = $root.find('.J_played_progress');
        this.$playStatus = $root.find('.J_play_status');
        this.$progressPanel = $root.find('.J_progress_panel');

        var audioEl = this.audioElement = document.createElement('audio');

        var playable = audioEl.canPlayType(o.mediaType);
        if (!playable || playable == "") {
            this.destroy();
            throw "can not play the music which mediaType is " + o.mediaType;
        }

        audioEl.src = o.mediaSRC;
        if (o.auto) {
            audioEl.autoplay = true;
        }
        if (o.loop) {
            audioEl.loop = true;
        }
        if (o.preload) {
            audioEl.preload = o.preload;
        }
        if (o.volume != "" && o.volume * 1 == o.volume) {
            audioEl.volume = o.volume;
        }

        this.position = 0;
        this.duration = 0;


        this._addEvent();
    };

    AudioPlayer.prototype = {
        constructor: AudioPlayer,
        audioEventTypes: "canplay timeupdate abord loadedmetadata durationchange progress play pause ended",
        _addEvent: function () {
            var audioEl = this.audioElement;
            var audioEventTypes = this.audioEventTypes.split(" "),
                self = this;

            $.each(audioEventTypes, function (_, item) {
                audioEl.addEventListener(item, self, false);
            });

            this.$root.on('click', '.J_btn', function (e) {
                e.preventDefault();
                var isPlaying = !audioEl.paused;
                if (isPlaying) {
                    self.pause();
                } else {
                    self.play();
                }
            });

            var $progressPanel = this.$progressPanel;

            var totalWidth = $progressPanel.width(),
                totalOffset = $progressPanel.offset(),
                $dom = $(document.body);

            var handleMove = function (e) {
                var percent = 0,
                    pos = 0,
                    x = 0,
                    originalEvent = e.originalEvent;

                if (originalEvent && originalEvent.changedTouches) {
                    x = originalEvent.changedTouches[0].pageX;
                } else if (e.changedTouches) {
                    x = e.changedTouches[0].pageX;
                } else {
                    x = e.pageX;
                }

                if (self.duration) {
                    if (x < totalOffset.left) {
                        x = totalOffset.left;
                    } else if (x > totalWidth + totalOffset.left) {
                        x = totalWidth + totalOffset.left;
                    }

                    pos = x - totalOffset.left;
                    percent = (pos / totalWidth);

                    if (self.mouseIsDown && pos != self.position) {
                        self.__moveProgress = percent;
                        self._setPlayedProgress(percent * 100);

                    } else {
                        self.__moveProgress = -1;
                    }
                }
            }

            $progressPanel.on('mousedown touchstart', function (e) {
                self.mouseIsDown = true;


                handleMove(e);
                $dom.on('mousemove.dur touchmove.dur', function (e) {
                    handleMove(e);
                }).on('mouseup.dur touchend.dur', function () {
                    self.mouseIsDown = false;
                    $dom.off('.dur');
                    if (self.__moveProgress >= 0 && self.__moveProgress != self.position && self.duration) {
                        var newTime = self.__moveProgress <= 0.02 ? 0 : self.__moveProgress * self.duration;
                        self.setCurrentTime(newTime);
                    }
                })
            });

        },
        _offEvent: function () {
            var audioEl = this.audioElement;
            var audioEventTypes = this.audioEventTypes.split(" "),
                self = this;

            $.each(audioEventTypes, function (_, item) {
                audioEl.removeEventListener(item, self, false);
            });
            this.$progressPanel.off();
            this.$root.off();
        },
        handleEvent: function (ev) {
            var type = ev.type,
                audioEvent = this._audioEvent[type];
            audioEvent && audioEvent.call(this, ev);
        },
        setCurrentTime: function (time) {
            if (time >= 0 && time <= this.duration) {
                this.audioElement.currentTime = time;
                this.position = this.audioElement.currentTime;
            }
            return this;
        },
        _audioEvent: {
            canplay: function (ev) {
                var o = this.opts;
                if (o.start) {
                    this.setCurrentTime(o.start);
                    o.start = 0;
                }
                if (o.auto) {
                    this.play();
                }
            },
            timeupdate: function (ev) {
                var audioEl = this.audioElement,
                    self = this;
                try {
                    this.position = audioEl.currentTime;
                    this.duration = audioEl.duration === Infinity ? null : audioEl.duration
                } catch (ex) {
                }

                this._setPlayedTime();
                this._setTotalTime();
                if (!self.mouseIsDown) {
                    this._setPlayedProgress();
                }

            },
            play: function (ev) {
                this.$playStatus.removeClass('icon-audio-play').addClass('icon-audio-pause');
            },
            pause: function (ev) {
                this.$playStatus.removeClass('icon-audio-pause').addClass('icon-audio-play');
            },
            loadedmetadata: function (ev) {
                var audioEl = this.audioElement,
                    o = this.opts;
                try {
                    this.duration = audioEl.duration === Infinity ? null : audioEl.duration
                } catch (ex) {
                }

                this._setTotalTime();
                if (o.start) {
                    this.setCurrentTime(o.start);
                    this._setPlayedTime();
                    this._setPlayedProgress();
                } else {
                    this._setPlayedTime(0);
                }
            },
            durationchange: function (ev) {
                var audioEl = this.audioElement,
                    o = this.opts;
                try {
                    this.duration = audioEl.duration === Infinity ? null : audioEl.duration
                } catch (ex) {
                }
                this._setTotalTime();
            },
            progress: function () {
                var audioEl = this.audioElement;

                if (audioEl && audioEl.buffered !== null && audioEl.buffered.length) {
                    this.duration = audioEl.duration === Infinity ? null : audioEl.duration;
                    this.loadPercent = parseInt((audioEl.buffered.end(audioEl.buffered.length - 1) / this.duration) * 100, 10);
                    if (this.loadPercent >= 100) {
                        this._clearLoadProgress();
                    } else {
                        this._setLoadProgress();
                    }
                }
            },
            ended: function () {
                this._setPlayedProgress(0);
                this._setPlayedTime(0);
            }
        },
        _clearLoadProgress: function () {
            this._setLoadProgress(0);
        },
        _setLoadProgress: function (loadPercent) {
            if (!loadPercent && loadPercent != 0) {
                loadPercent = this.loadPercent;
            }

            this.$loadedProgress.width(loadPercent + '%');
        },
        _setPlayedProgress: function (percent) {
            if (!percent && percent != 0) {
                percent = (this.position / this.duration) * 100;
            }
            this.$playedProgress.width(percent + "%");
        },
        _setPlayedTime: function (time) {
            if (!time && time != 0) {
                time = this.position;
            }
            var played = playTimeTransform(time);
            this.$playedTime.text(played);
        },
        _setTotalTime: function (time) {
            if (!time && time != 0) {
                time = this.duration;
            }
            var total = playTimeTransform(time);
            this.$totalTime.text(total);
        },
        getAudioElement: function () {
            return this.audioElement;
        },
        play: function () {
            var audioEl = this.audioElement;
            if (audioEl.paused) {
                audioEl.play();
            }
            return this;
        },
        pause: function () {
            var audioEl = this.audioElement;
            if (!audioEl.paused) {
                audioEl.pause()
            }
            return this;
        },
        muted: function (bool) {
            this.audioElement.muted = !!bool;
            return this;
        },
        destroy: function () {
            this.$root.remove();
            this._offEvent();
            var self = this;
            $.each(self, function (key) {
                if (self.hasOwnProperty(key)) {
                    self[key] = null;
                    delete self[key];
                }
            });
            self = null;
        }
    };

    AudioPlayer.DEF = {
        auto: true,
        loop: false,
        preload: 'auto',
        start: 0,
        mediaSRC: "",
        mediaType: "audio/mp3",
        volume: 1,
        parentNode: document.body
    };

    //helper

    function timeFixed(num) {
        num = Math.abs(num);
        if (num >= 10) {
            return num;
        }
        return "0" + num;
    }

    function playTimeTransform(second) {
        if (!second && second != 0) {
            return "--:--";
        }
        var mm = Math.floor(second / 60),
            ss = Math.floor(second % 60);
        return [timeFixed(mm), timeFixed(ss)].join(":");
    }

    module.exports = AudioPlayer;

});

define("player.video", ["jquery"], function (require, exports, module) {
    "use strict";
    var $ = require("jquery");
    
    var VideoPlayer = function (opts) {
        
    };
    
    VideoPlayer.prototype = {
        constructor: VideoPlayer
    };
    
    VideoPlayer.DEF = {
        auto: true,
        loop: false,
        preload: 'auto',
        start: 0,
        mediaSRC: "",
        mediaType: "audio/mp3",
        volume: 1,
        parentNode: document.body,
        poster: "http://www.sinaimg.cn/cj/licaishi/image/20160401/h5_video_poster.jpg"
    }

});